###############################################################################
## TimeTravel by Zoomj [0.1]
##
## File: helpers.py
## Description: game helper functions
## Last modified: 03/07/2011

from __future__ import with_statement
import es
import usermsg
import cmdlib

""" Useage: from helpers import * """
__all__ = [
    'get_player_velocity',
    'set_player_velocity',
    'kill_player_velocity',
    'get_player_angles',
    'set_player_angles',
    'get_player_location',
    'set_player_location',
    'freeze_player',
    'unfreeze_player',
    'send_hint',
    'regcmd_c',
    'unregcmd_c',
]

###############################################################################
## Get/set and kill players velocity

""" Return players velocity as a vector (tuple style) """
def get_player_velocity(userid):
    x = y = z = 0.0
    try:
        x = float(es.getplayerprop(userid,
            'CBasePlayer.localdata.m_vecVelocity[0]'))
        y = float(es.getplayerprop(userid,
            'CBasePlayer.localdata.m_vecVelocity[1]'))
        z = float(es.getplayerprop(userid,
            'CBasePlayer.localdata.m_vecVelocity[2]'))
    except TypeError:
        pass
    return (x, y, z)
    
""" Set players velocity """
def set_player_velocity(userid, velocity_vec):
    new_velocity = es.createvectorstring(
        velocity_vec[0], velocity_vec[1], velocity_vec[2])
    es.setplayerprop(userid,
        'CBasePlayer.localdata.m_vecBaseVelocity', new_velocity)
    
""" Remove all velocity from player """
def kill_player_velocity(userid):
    x = y = z = 0.0
    try:
        x = float(es.getplayerprop(userid,
            'CBasePlayer.localdata.m_vecVelocity[0]'))
        y = float(es.getplayerprop(userid,
            'CBasePlayer.localdata.m_vecVelocity[1]'))
        z = float(es.getplayerprop(userid,
            'CBasePlayer.localdata.m_vecVelocity[2]'))
    except TypeError:
        pass
    es.setplayerprop(userid,
        'CBasePlayer.localdata.m_vecBaseVelocity',
        es.createvectorstring(x * -1.0, y * -1.0, z * -1.0))
    
###############################################################################
## Get and set players view/eye angles

""" Return players view angles as a vector (tuple style) """
def get_player_angles(userid):
    return (
        float(es.getplayerprop(userid,
            'CCSPlayer.m_angEyeAngles[0]')),
        float(es.getplayerprop(userid,
            'CCSPlayer.m_angEyeAngles[1]')), 0.0)
    
""" Set players view angle """
def set_player_angles(userid, angles_vec):
    es.server.cmd('es_xsetang %s %s %s %s' %
        (userid, angles_vec[0], angles_vec[1], angles_vec[2]))

###############################################################################
## Get and set players location/position
    
""" Return players location as a vector (tuple style) """
def get_player_location(userid):
    return es.getplayerlocation(userid)
    
""" Set players location """
def set_player_location(userid, loc_vec):
    es.server.cmd('es_xsetpos %s %s %s %s' %
        (userid, loc_vec[0], loc_vec[1], loc_vec[2]))

###############################################################################
## Freeze/unfreeze players movement
    
""" Freeze players movements completely """
def freeze_player(userid):
    es.setplayerprop(userid, 'CBaseEntity.movetype', '0')
    
""" Reset players movements back to default """
def unfreeze_player(userid):
    es.setplayerprop(userid, 'CBaseEntity.movetype', '2')

###############################################################################
## Hud-hint/chat message helpers

""" Send a generic hud-hint (HintText) message to player """
def send_hint(userid, message):
    es.usermsg('create', 'generic_msg', 'HintText')
    es.usermsg('write', 'byte', 'generic_msg', 1)
    es.usermsg('write', 'string', 'generic_msg', message)
    es.usermsg('send', 'generic_msg', userid)
    es.usermsg('delete', 'generic_msg')
    
###############################################################################
## Short wrappers for using cmdlib

""" Register a basic non-auth'd client command with cmdlib """
def regcmd_c(cmd, callback, desc = ''):
    try: cmdlib.registerClientCommand(cmd, callback, desc)
    except NameError: pass
    
""" Un-register a client command with cmdlib """
def unregcmd_c(command):
    cmdlib.unregisterClientCommand(command)
    
###############################################################################